//+------------------------------------------------------------------+
//|                                                       ADR+ST.mq4 |
//|                                     Copyright  2008, alex_smith |
//|                                              alex_smith@front.ru |
//|                               Average Day Range and Session Time |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, alex_smith"
#property link      "alex_smith@front.ru"

#property indicator_chart_window
extern int    NumberOfDays = 50;
extern string Begin_1      = "00:00";
extern string End_1        = "12:00";
extern color  Color_1      = Orange;
extern string Begin_2      = "10:00";
extern string End_2        = "18:00";
extern color  Color_2      = DeepSkyBlue;
extern string Begin_3      = "15:00";
extern string End_3        = "23:00";
extern color  Color_3      = LimeGreen;
extern int    ADR_Width    = 2; 
extern color  Color_Line1  = Red;
extern int    DR_Width     = 2; 
extern color  Color_Line2  = DarkBlue;
double tic;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   if(Period()>240) return(0);
   tic=MarketInfo(Symbol(),MODE_TICKSIZE);
   DeleteObjects();
    CreateObjects("ADRH",Color_Line1,OBJ_HLINE);
    CreateObjects("ADRL",Color_Line1,OBJ_HLINE);
    CreateObjects("DRH",Color_Line2,OBJ_HLINE);
    CreateObjects("DRL",Color_Line2,OBJ_HLINE);
   for(int i=0;i<NumberOfDays;i++) {
    CreateObjects("ST1"+i,Color_1,OBJ_RECTANGLE);
    CreateObjects("ST2"+i,Color_2,OBJ_RECTANGLE);
    CreateObjects("ST3"+i,Color_3,OBJ_RECTANGLE);
    }
   Comment("");
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
   DeleteObjects();
   Comment("");
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   double sum,DayMax,DayMin;  
   datetime dt=TimeCurrent();
   for (int i=0; i<NumberOfDays; i++) {
    sum+=iHigh(Symbol(),PERIOD_D1,i)-iLow(Symbol(),PERIOD_D1,i);
    DrawRectangle(dt, "ST1"+i, Begin_1, End_1);
    DrawRectangle(dt, "ST2"+i, Begin_2, End_2);
    DrawRectangle(dt, "ST3"+i, Begin_3, End_3);
    dt=decDateTradeDay(dt);
    while (TimeDayOfWeek(dt)==6||TimeDayOfWeek(dt)==0) dt=decDateTradeDay(dt);
  }
   int sumall=sum/NumberOfDays/tic;
   DayMax=iHigh(Symbol(),PERIOD_D1,0);
   DayMin=iLow(Symbol(),PERIOD_D1,0);
   int sumtec=(DayMax-DayMin)/tic;
   double koef=NormalizeDouble(sum/NumberOfDays/(DayMax-DayMin),2);
   Comment("Average daily range for "+NumberOfDays+" days : "+sumall+" points.","\n",
   "Current daily range: "+sumtec+" points.","\n",
   "Average to current rate "+koef);
   double prh=iOpen(Symbol(),PERIOD_D1,0)+sumall*tic/2;
   double prl=iOpen(Symbol(),PERIOD_D1,0)-sumall*tic/2;
   if(DayMin<prl) {prh=DayMax; prl=DayMax-sumall*tic;}
   if(DayMax>prh) {prl=DayMin; prh=DayMin+sumall*tic;}
   ObjectSet("ADRH",OBJPROP_PRICE1,prh);
   ObjectSet("ADRH",OBJPROP_WIDTH,ADR_Width);
   ObjectSet("ADRH",OBJPROP_BACK,True);
   ObjectSet("ADRL",OBJPROP_PRICE1,prl);
   ObjectSet("ADRL",OBJPROP_WIDTH,ADR_Width);
   ObjectSet("ADRL",OBJPROP_BACK,True);
   ObjectSet("DRH",OBJPROP_PRICE1,DayMax);
   ObjectSet("DRH",OBJPROP_WIDTH,ADR_Width);
   ObjectSet("DRH",OBJPROP_BACK,True);
   ObjectSet("DRL",OBJPROP_PRICE1,DayMin);
   ObjectSet("DRL",OBJPROP_WIDTH,ADR_Width);
   ObjectSet("DRL",OBJPROP_BACK,True);
   return(0);
  }
//+------------------------------------------------------------------+
//|                                        |
//+------------------------------------------------------------------+
void DeleteObjects() {
    ObjectDelete("ADRH");
    ObjectDelete("ADRL");
    ObjectDelete("DRH");
    ObjectDelete("DRL");
    for (int i=0; i<NumberOfDays; i++) {
    ObjectDelete("ST1"+i);
    ObjectDelete("ST2"+i);
    ObjectDelete("ST3"+i);
  }
}
//+------------------------------------------------------------------+
//|                                        |
//| :                                                       |
//|   no -                                        |
//|   cl -                                                |
//+------------------------------------------------------------------+
void CreateObjects(string no,color cl,int iObject) {
  ObjectCreate(no,iObject,0,0,0,0,0);
  ObjectSet(no, OBJPROP_STYLE, STYLE_SOLID);
  ObjectSet(no, OBJPROP_COLOR, cl);
  ObjectSet(no, OBJPROP_BACK, True);
}
//+------------------------------------------------------------------+
//|                                       |
//| :                                                       |
//|   dt -                                           |
//|   no -                                        |
//|   tb -                                          |
//|   te -                                       |
//+------------------------------------------------------------------+
void DrawRectangle(datetime dt, string no, string tb, string te) {

  datetime t1=StrToTime(TimeToStr(dt, TIME_DATE)+" "+tb);
  datetime t2=StrToTime(TimeToStr(dt, TIME_DATE)+" "+te);
  int b1=iBarShift(Symbol(),0,t1);
  int b2=iBarShift(Symbol(),0,t2);
  double p1=High[Highest(Symbol(),0,MODE_HIGH,b1-b2,b2)];
  double p2=Low [Lowest (Symbol(),0,MODE_LOW ,b1-b2,b2)];
  ObjectSet(no,OBJPROP_TIME1,t1);
  ObjectSet(no,OBJPROP_PRICE1,p1);
  ObjectSet(no,OBJPROP_TIME2,t2);
  ObjectSet(no,OBJPROP_PRICE2,p2);
}
//+------------------------------------------------------------------+
//|                                  |
//| :                                                       |
//|   dt -                                           |
//+------------------------------------------------------------------+
datetime decDateTradeDay (datetime dt) {
  int ty=TimeYear(dt);
  int tm=TimeMonth(dt);
  int td=TimeDay(dt);
  int th=TimeHour(dt);
  int ti=TimeMinute(dt);

  td--;
  if (td==0) {
    tm--;
    if (tm==0) {
      ty--;
      tm=12;
    }
    if (tm==1 || tm==3 || tm==5 || tm==7 || tm==8 || tm==10 || tm==12) td=31;
    if (tm==2) if (MathMod(ty, 4)==0) td=29; else td=28;
    if (tm==4 || tm==6 || tm==9 || tm==11) td=30;
  }
  return(StrToTime(ty+"."+tm+"."+td+" "+th+":"+ti));
}
//+------------------------------------------------------------------+